#include <stdio.h>
#include <iostream>


#include "TApplication.h"
#include "TROOT.h"
#include "TVector3.h"
#include "TStyle.h"

#include <TEveManager.h>
#include <TEveCalo.h>
#include <TEveCaloData.h>
#include <TEveSelection.h>
#include "TEveGeoNode.h"
#include "TEveProjections.h"
#include "TEveProjectionAxes.h"
#include "TEveProjectionManager.h"
#include "TEveViewer.h"
#include "TEveScene.h"
#include "TEveLine.h"

#include "TGLViewer.h"
#include "TGeoMatrix.h"
#include "TGeoSphere.h"
#include "TGeoTube.h"


class VizCluster : public TEveGeoShape {
  public:
    VizCluster(double energy, double pt, double eta, double phi, double dlambda, double dr);
    ~VizCluster() { }
  private:
    double m_ClusEtMin;
};

VizCluster::VizCluster(double energy, double _pt, double eta, double phi, double _dlambda, double _dr)
                              : TEveGeoShape("cls","cls"), m_ClusEtMin(0.5e3)
{
  double pt = _pt/10.;
  double dlambda = _dlambda/10.;
  double dr = _dr/10.;
  std::cout << "VizCluster() -> " << "e:" << energy << " pt:" << pt << " eta:" << eta << " phi:" << phi << "dlambda:" << dlambda << " dr:" << dr << std::endl;
  TVector3 v;
  v.SetPtEtaPhi(pt, eta, phi);
  TGeoSphere *sphere = new TGeoSphere("sph", 0.0, dr);
  TGeoScale *scale = new TGeoScale(1.0, 1.0, dlambda/dr);
  SetMainTransparency(10);
  int icolor = (int)(log10(energy/m_ClusEtMin)/2.5*50.+51.);
  if ( icolor > 100 ) icolor = 100;
  SetMainColor(icolor);
  TGeoTranslation *t1 = new TGeoTranslation(v.X(),v.Y(),v.Z());
  TGeoRotation *r1 = new TGeoRotation("rot");
  float xt = v.Theta()*180./M_PI;
  float xphi = v.Phi()*180./M_PI;
  r1->SetAngles(90.,0.-xphi,  90.+xt,90.-xphi,  0.+xt,90.-xphi);
  SetTransMatrix( (*t1)*(*r1)*(*scale));
  SetShape(sphere);
//  sh->SetNSegments(100);
}


void update3d();
void init3d();
TEveProjectionManager* m_RhoZMgr = 0;
TEveProjectionManager* m_RPhiMgr = 0;



/* ****************************************************************************

**************************************************************************** */
int main()
{
  gStyle->SetCanvasColor(10);
  gStyle->SetCanvasBorderMode(0);
  gStyle->SetPadColor(10);
  gStyle->SetPadBorderMode(0);
  gStyle->SetFrameFillColor(10);

  TApplication theApp("theApp",0,0);


  // --- test eve -------------
  
  // to draw simple geometry
  init3d();
  update3d();

  // to draw one cluster
  TEveElementList* lClus = new TEveElementList("Clusters");
  lClus->SetTitle("Clusters");
  gEve->AddElement(lClus);
  VizCluster *v = new VizCluster(50000., 2000., 1.5, M_PI/4., 500., 300.);
  gEve->AddElement(v, lClus);

  update3d();
  // ---------------------------

  std::cout << std::endl << "We reach the end of the main(), keep holding your graphics..." << std::endl;
  theApp.Run();
  return 0;
}



/* ****************************************************************************

**************************************************************************** */
void init3d()
{
  if( !gEve ) {
    TEveManager::Create();
    gEve->GetSelection()->SetPickToSelect(1);

    // defining simple calorimeters
    struct MyCaloData {
      const char *title;
      float r1;
      float r2;
      float dzhalf;
      float zshift;
      int color;
    };
    MyCaloData CaloData[] = {
      {"emb",   140.,  203., 300., 0,     kBlue},
      {"hecl",   47.5, 203., 92.5, -519.5,  kOrange},
      {"hecr",   47.5, 203., 92.5, 519.5,   kOrange}
    };

    // XY projection
    TEveScene*  s1 = gEve->SpawnNewScene("XY Projection");
    TEveViewer* v1 = gEve->SpawnNewViewer("XYViewer");
    v1->AddScene(s1);
    TGLViewer* v1gl = v1->GetGLViewer();
    v1gl->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
    v1gl->SetGuideState(TGLUtil::kAxesOrigin, kTRUE, kFALSE, 0);
    v1gl->SetClearColor(kBlue + 4);
    m_RPhiMgr = new TEveProjectionManager();
    m_RPhiMgr->SetProjection(TEveProjection::kPT_RPhi);
    gEve->AddElement(m_RPhiMgr, s1);
    gEve->AddToListTree(m_RPhiMgr, kTRUE);
    TEveProjectionAxes* a1 = new TEveProjectionAxes(m_RPhiMgr);
    s1->AddElement(a1);
    gEve->AddToListTree(a1, kTRUE);
//     m_RPhiMgr->ImportElements((TEveElement *)gEve->GetGlobalScene());
//     m_RPhiMgr->ImportElements((TEveElement *)gEve->GetEventScene());

    // --- Rho-Z projection
    TEveViewer* v2 = gEve->SpawnNewViewer("Rho-Z", false);
    TEveScene*  s2 = gEve->SpawnNewScene("Rho-Z Projection");
    v2->AddScene(s2);
    TGLViewer* v2gl = v2->GetGLViewer();
    v2gl->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
    v2gl->SetGuideState(TGLUtil::kAxesOrigin, kTRUE, kFALSE, 0);
    v2gl->SetClearColor(kBlue + 4);
    m_RhoZMgr = new TEveProjectionManager();
    m_RhoZMgr->SetProjection(TEveProjection::kPT_RhoZ);
    gEve->AddElement(m_RhoZMgr, s2);
    gEve->AddToListTree(m_RhoZMgr, kTRUE);
    TEveProjectionAxes* a2 = new TEveProjectionAxes(m_RhoZMgr);
    s2->AddElement(a2);
    gEve->AddToListTree(a2, kTRUE);
//     m_RhoZMgr->ImportElements((TEveElement *)gEve->GetGlobalScene());
//     m_RhoZMgr->ImportElements((TEveElement *)gEve->GetEventScene());

    TEveElementList* lCalo = new TEveElementList("Calorimeters");
    lCalo->SetTitle("Calorimeters");
    gEve->AddElement(lCalo);
    for(int i_mod=0; i_mod<int(sizeof(CaloData)/sizeof(MyCaloData)); i_mod++) {
      std::cout << " i_mod:" << i_mod << std::endl;
      TGeoTube *vol = new TGeoTube(CaloData[i_mod].title, CaloData[i_mod].r1, CaloData[i_mod].r2, CaloData[i_mod].dzhalf);
      TEveGeoShape *sh = new TEveGeoShape(CaloData[i_mod].title,CaloData[i_mod].title);
      if(CaloData[i_mod].zshift != 0) {
        TGeoTranslation *t1 = new TGeoTranslation(0., 0., CaloData[i_mod].zshift);
        sh->SetTransMatrix( *t1 );
      }
      sh->SetMainTransparency(70);
      sh->SetMainTransparency(70);
      sh->SetMainColor(CaloData[i_mod].color);
      sh->SetShape(vol);
      lCalo->AddElement(sh);
    }
    m_RPhiMgr->ImportElements(lCalo);
    m_RhoZMgr->ImportElements(lCalo);
    gEve->Redraw3D();
  }
}


/* ****************************************************************************

**************************************************************************** */
void update3d()
{
  TEveElement* top = (TEveElement *)gEve->GetCurrentEvent();
  if(top){
    m_RPhiMgr->DestroyElements();
    m_RPhiMgr->ImportElements(top);
    m_RhoZMgr->DestroyElements();
    m_RhoZMgr->ImportElements(top);
  }
  gEve->Redraw3D(1);
}

